module Statham
  # Public: Collection of definition of child attributes inside JSON.
  class AttributeSet
    # Public: Name of the parent JSON attribute.
    attr_reader :name
    # Public: AttributeSet defined in the parent class of the owner of the
    # AttributeSet.
    attr_reader :parent

    # Internal: Initialize an AttributeSet.
    #
    # options - The hash options used to define AttributeSets:
    #           :name - Name of the parent attribute.
    #
    # Returns new AttributeSet.
    def initialize(options = {})
      @name = options[:name]
      @parent = options[:parent]
      @attributes = {}
    end

    # Public: Define an attribute serialized inside JSON.
    #
    # name    - Name of the attribute.
    # options - The hash options used to define new attribute:
    #           :type    - Type of the attribute.
    #           :default - Default value for the attribute.
    #
    # Returns nothing.
    def attribute(name, options = {})
      @attributes[name] = Statham::Attribute.new(options)
    end

    # Public: Collection of attributes definition. If parent AttributeSet
    # exists, it merges attributes on the AttributeSet with attributes of
    # parent.
    #
    # Returns Hash collection of attributes.
    def attributes
      if @parent
        @attributes.reverse_merge @parent.attributes
      else
        @attributes
      end
    end
  end
end
